@@ -28,6 +28,9 @@ module Codec.Serialise.Class
  , GSerialiseSum(..)
  , encodeVector
  , decodeVector
+ , encodeContainerSkel
+ , encodeMapSkel
+ , decodeMapSkel
  ) where
 
 import           Control.Applicative
@@ -856,9 +859,10 @@ instance Serialise a => Serialise (Tree.Tree a) where
   encode (Tree.Node r sub) = encodeListLen 2 <> encode r <> encode sub
   decode = decodeListLenOf 2 *> (Tree.Node <$> decode <*> decode)
 
-encodeContainerSkel :: (Word -> Encoding)
-                    -> (container -> Int)
-                    -> (accumFunc -> Encoding -> container -> Encoding)
+-- | Patch functions together to obtain an 'Encoding' for a container.
+encodeContainerSkel :: (Word -> Encoding) -- ^ encoder of the length
+                    -> (container -> Int) -- ^ length
+                    -> (accumFunc -> Encoding -> container -> Encoding) -- ^ foldr
                     -> accumFunc
                     -> container
                     -> Encoding
@@ -996,8 +1000,9 @@ instance (Serialise a, Hashable a, Eq a) => Serialise (HashSet.HashSet a) where
   encode = encodeSetSkel HashSet.size HashSet.foldr
   decode = decodeSetSkel HashSet.fromList
 
+-- | A helper function for encoding maps.
 encodeMapSkel :: (Serialise k, Serialise v)
-              => (m -> Int)
+              => (m -> Int) -- ^ obtain the length
               -> ((k -> v -> Encoding -> Encoding) -> Encoding -> m -> Encoding)
               -> m
               -> Encoding
@@ -1009,8 +1014,9 @@ encodeMapSkel size foldrWithKey =
     (\k v b -> encode k <> encode v <> b)
 {-# INLINE encodeMapSkel #-}
 
+-- | A utility function to construct a 'Decoder' for maps.
 decodeMapSkel :: (Serialise k, Serialise v)
-              => ([(k,v)] -> m)
+              => ([(k,v)] -> m) -- ^ fromList
               -> Decoder s m
 decodeMapSkel fromList = do
   n <- decodeMapLen
@@ -1146,6 +1152,16 @@ instance Serialise RuntimeRep where
       AddrRep       -> encodeListLen 1 <> encodeWord 9
       FloatRep      -> encodeListLen 1 <> encodeWord 10
       DoubleRep     -> encodeListLen 1 <> encodeWord 11
+#if MIN_VERSION_base(4,13,0)
+      Int8Rep       -> encodeListLen 1 <> encodeWord 12
+      Int16Rep      -> encodeListLen 1 <> encodeWord 13
+      Word8Rep      -> encodeListLen 1 <> encodeWord 14
+      Word16Rep     -> encodeListLen 1 <> encodeWord 15
+#endif
+#if MIN_VERSION_base(4,14,0)
+      Int32Rep      -> encodeListLen 1 <> encodeWord 16
+      Word32Rep     -> encodeListLen 1 <> encodeWord 17
+#endif
 
   decode = do
     len <- decodeListLen
@@ -1163,6 +1179,16 @@ instance Serialise RuntimeRep where
       9  | len == 1 -> pure AddrRep
       10 | len == 1 -> pure FloatRep
       11 | len == 1 -> pure DoubleRep
+#if MIN_VERSION_base(4,13,0)
+      12 | len == 1 -> pure Int8Rep
+      13 | len == 1 -> pure Int16Rep
+      14 | len == 1 -> pure Word8Rep
+      15 | len == 1 -> pure Word16Rep
+#endif
+#if MIN_VERSION_base(4,14,0)
+      16 | len == 1 -> pure Int32Rep
+      17 | len == 1 -> pure Word32Rep
+#endif
       _             -> fail "Data.Serialise.Binary.CBOR.getRuntimeRep: invalid tag"
 
 -- | @since 0.2.0.0
@@ -1210,11 +1236,11 @@ decodeSomeTypeRep = do
     case tag of
       0 | len == 1 ->
               return $! SomeTypeRep (typeRep :: TypeRep Type)
-      1 | len == 2 -> do
+      1 | len == 3 -> do
               !con <- decode
               !ks <- decode
               return $! SomeTypeRep $ mkTrCon con ks
-      2 | len == 2 -> do
+      2 | len == 3 -> do
               SomeTypeRep f <- decodeSomeTypeRep
               SomeTypeRep x <- decodeSomeTypeRep
               case typeRepKind f of
@@ -1233,7 +1259,7 @@ decodeSomeTypeRep = do
                      [ "Applied type: " ++ show f
                      , "To argument:  " ++ show x
                      ]
-      3 | len == 2 -> do
+      3 | len == 3 -> do
               SomeTypeRep arg <- decodeSomeTypeRep
               SomeTypeRep res <- decodeSomeTypeRep
               case typeRepKind arg `eqTypeRep` (typeRep :: TypeRep Type) of
@@ -1242,7 +1268,9 @@ decodeSomeTypeRep = do
                       Just HRefl -> return $! SomeTypeRep $ Fun arg res
                       Nothing -> failure "Kind mismatch" []
                 Nothing -> failure "Kind mismatch" []
-      _ -> failure "unexpected tag" []
+      _ -> failure "unexpected tag"
+           [ "Tag: " ++ show tag
+           , "Len: " ++ show len ]
   where
     failure description info =
         fail $ unlines $ [ "Codec.CBOR.Class.decodeSomeTypeRep: "++description ]
@@ -1268,7 +1296,6 @@ encodeTypeRep (Fun arg res)
  <> encodeWord 3
  <> encodeTypeRep arg
  <> encodeTypeRep res
-encodeTypeRep _ = error "Codec.CBOR.Class.encodeTypeRep: Impossible"
 
 -- | @since 0.2.0.0
 instance Typeable a => Serialise (TypeRep (a :: k)) where
